%% Mrsi adatok betltse
% A kvetkez parancssorban a <neptun> helyre rja be sajt Neptun-kdjt,
% pl. load abc123 meres_u meres_y;
% A parancs a <neptun>.mat fjlbl betlti a mrsi regisztrtumot,
% a mrs sorn hasznlt bemen jelet a meres_u, a mrt kimenetet
% a meres_y vltozba tlti
close all
clear all
load slz0ue meres_u meres_y;
% Ts az a mintavteli id, amit az identifikci s a 
% szablyoz megtervezse sorn kell hasznlnia
Ts=0.1;

%% Identifikci
% Az identifikci vgeredmnyt a kvetkez vltozkban trolja:
% Ap - a szakasz tviteli fggvnynek Ap paramtere
% xi - a szakasz tviteli fggvnynek xi paramtere
% Tau - a szakasz tviteli fggvnynek Tau paramtere
% alpha - a szakasz tviteli fggvnynek alpha paramtere
z=iddata(meres_y,meres_u,Ts);
nA=3;   % nevezo fokszama
nk=1;   % holtido fokszama
nB=nA-1+nk;
nC=nA;
tharmax=armax(z,[nA nB nC nk]); % th struktura
[A,B,C]=th2poly(tharmax);
Darmax=tf(B,A,Ts);
t=(0:Ts:length(meres_u)*Ts-Ts);
yid=lsim(Darmax,meres_u,t);

set(0,'defaultlinelinewidth', 1);
figure(1);
plot(t,meres_y,'g',t,yid,'r');
legend('Mrt vlaszjel','Az identifiklt rendszer vlasza');
xlabel('Time (seconds)','FontSize',12);
ylabel('y(t), yid(t)','FontSize',12);
ylim([-10 13]);
grid on

zpoles=roots(A);                % Diszkret ideju polusok szamitasa
spoles=log(zpoles)/Ts;          % A folytonos ideju polusok szamitasa
%Tcs=-1./spoles;                 % Az idllandok
spoly=poly(spoles);
D0=polyval(B,1)/polyval(A,1);   % A diszkret ideju atvitel allandosult allapotban
Ap=D0               % Allandosult allapotban az erosites
Warmax=tf(Ap*spoly(end),spoly);            % Folytonos atvit
zpkW=zpk(Warmax);               % Folytonos atvit zpkban
Tau=sqrt(1/(zpkW.p{1}(3)* zpkW.p{1}(2)))
alpha=(1/(-1*zpkW.p{1}(1)))/Tau
%Ap=Warmax.num{1}(4)/Warmax.den{1}(4)
omega_0=1/Tau;
%Sigma_e=-((zpkW.p{1}(3)+ zpkW.p{1}(2))/2)
Sigma_e=-max(real(zpkW.p{1}));
xi=Sigma_e/omega_0
%xi=-(((zpkW.p{1}(3)+ zpkW.p{1}(2))/2)/(zpkW.p{1}(3)* zpkW.p{1}(2)))/Tau


%% Szablyoztervezs
% A szablyoz polinomjait a kvetkez vltozkban trolja:
% R - a szablyoz R(z) polinomja
% S - a szablyoz S(z) polinomja
% T - a szablyoz T(z) polinomja
xi_spec=0.7;
w0=1/Tau;
sdom1=-w0*xi_spec+1j*w0*sqrt(1-xi_spec^2);
sdom2=conj(sdom1);
scinf=-3*abs(real(sdom1));
soinf=-5*abs(real(sdom1));
lint=1;                         % Integratorok szama
DWarmax=c2d(Warmax, Ts, 'zoh');
zdom1=exp(sdom1*Ts);
zdom2=exp(sdom2*Ts);
zcinf=exp(scinf*Ts);
zoinf=exp(soinf*Ts);

figure(2);
pzmap(DWarmax);

B=DWarmax.num{1};           % B kinyerese
B=B(2:end);                 % Vezeto nullak elhagyas
Broots=roots(B);
Bminus=B;                   % B faktorizacio, Bminus a kiejtheto zerusok
Bplus=1;                    % B faktorizacio, Bplus a nem kiejtheto zerusok
grB=length(B)-1;            % B fokszama, az egyutthatoi szamanal 1-el kisebb

A=DWarmax.den{1};           % A kinyerese
grA=length(A)-1;            % A fokszama, az egyutthatoi szamanal 1-el kisebb
grBminus=length(Bminus)-1;  % Bminus fokszama, az egyutthatoi szamanal 1-el kisebb

%Fokszam feltetelek
grAm=1+grBminus;            % Mivel grBminus>1, ezert nincs plusz tag
grS=grA+lint-1;
grA0=grA+lint-1;            % Nincs plusz tag
grR1prime=grBminus;

%Am es Ao polinomok kiszamitasa
Am=poly([zdom1 zdom2 zcinf]);     % Am gyokei a dominans poluspar es zcinf
A0=poly(ones(1,grA0)*zoinf);      % Ao gyokei zoinf, grAo-szorosa

% Az egyenletrendszer dioA*x=dioC alaku
polyint=[1 -1];                     % (1-z)^lint
Atilde=conv(polyint,A);
dioA=[ toeplitz([Atilde 0],[Atilde(1) 0]) toeplitz([Bminus 0 0 0],[Bminus(1) 0 0 0]) ];
Ctilde=conv(Am,A0)-[Atilde 0 0];
dioC=Ctilde(2:end)';
sol=inv(dioA)*dioC;                 % Az egyenletrendszer megoldasa

R1prime=[1 sol(1:2)'];              % R1prime monic, igy polinomjanak legmagasabb fokszamu egyutthatoja 1, 
                                    % csak utana kovetkeznek a kiszamitott egyutthatok
R=conv(R1prime,polyint);            % R=R1prime*(z-1), mivel lint=1 s Bplus=1
S=sol(3:end);                       % S polinom egyutthatoi a megoldasvektor tovabbi elemei
S=S';

% A T polinom szmtsa
Bmprime=polyval(Am,1)/polyval(Bminus,1);
T=Bmprime*A0;

% A szabalyozo tagok atviteli fuggvenyei
d_ff=tf(T,R,Ts);                    % Elorecsatolo ag
d_fb=tf(S,R,Ts);                    % Visszacsatol ag
dyr=d_ff*feedback(DWarmax,d_fb,-1); % A zart kor r->y atviteli fuggvenye

figure(3);
step(dyr);
title('A zrt kr ugrsvlasza','FontSize',10);
xlabel('Time','FontSize',12);
ylabel('y(t)','FontSize',12);
grid on

% A szakasz minden parameteret 75%-ra csokkentjuk
Ap75=Ap*0.75;
xi75=xi*0.75;
Tau75=Tau*0.75;
alpha75=alpha*0.75;
DEN75=conv([Tau75^2 2*xi75*Tau75 1],[alpha75*Tau75 1]);
wp75=tf(Ap75,DEN75);
% A szakasz minden parameteret 125%-ra noveljuk
Ap125=Ap*1.25;
xi125=xi*1.25;
Tau125=Tau*1.25;
alpha125=alpha*1.25;
DEN125=conv([Tau125^2 2*xi125*Tau125 1],[alpha125*Tau125 1]);
wp125=tf(Ap125,DEN125);
% Atteres diszkret idore
DWarmax75=c2d(wp75,Ts,'zoh');
DWarmax125=c2d(wp125,Ts,'zoh');
% Robusztussag vizsgalata
dur=d_ff*feedback(tf(1,1,Ts),DWarmax*d_fb,-1);
dur75=d_ff*feedback(tf(1,1,Ts),DWarmax75*d_fb,-1);
dur125=d_ff*feedback(tf(1,1,Ts),DWarmax125*d_fb,-1);

figure(4);
step(dur, dur75, dur125);
legend('100%', '75%', '125%');
title('A beavatkoz jel zrt krben','FontSize',10);
xlabel('Time','FontSize',12);
ylabel('u(t)','FontSize',12);
grid on

% Zart kor osszeallitasa
dyr75=d_ff*feedback(DWarmax75,d_fb,-1);
dyr125=d_ff*feedback(DWarmax125,d_fb,-1);

figure(5);
step(dyr,dyr75,dyr125);
legend('100%', '75%', '125%');
title('A 2DOF szablyz robusztussgnak illusztrcija','FontSize',10);
xlabel('Time','FontSize',12);
ylabel('y(t)','FontSize',12);
grid on
